@@ -1234,7 +1234,7 @@ public:
     QVector<QByteArray> errorCodes;
 };
 
-class Extensions
+class KWIN_EXPORT Extensions
 {
 public:
     bool isShapeAvailable() const {
